local function am(msg)
	ChatFrame1:AddMessage(msg);
end

-------------------------------------------------------------------------------------------------------------------------
local _name, _addon = ...;

local SOUNDS = {
	"Sound\\Creature\\GoblinMaleZanyNPC\\GoblinMaleZanyNPCGreeting01.wav",
	"Sound\\Creature\\GoblinFemaleZanyNPC\\GoblinFemaleZanyNPCGreeting04.wav",
	"Sound\\Creature\\GoblinMaleGruffNPC\\GoblinMaleGruffNPCGreeting01.wav",
	"Sound\\Creature\\GoblinMaleGuardNPC\\GoblinMaleGuardNPCGreeting03.wav",
};

local BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	edgeSize = 16,
	insets = { top = 2, right = 2, left = 2, bottom = 2, },
};

-------------------------------------------------------------------------------------------------------------------------
local flag = {};
local function _scanMerchantCurrency()
	local currencies = { GetMerchantCurrencies() };

	wipe(flag);
	local items = {};
	for i = 1, GetMerchantNumItems() do
		local itemCount = GetMerchantItemCostInfo(i);
		for j = 1, itemCount do
			local link = select(3, GetMerchantItemCostItem(i, j));
			if link then
				--item cost, parse itemID
				local _, _, _, _, itemID = string.find(link, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?");
				if not flag[itemID] then
					flag[itemID] = 1;
					tinsert(items, itemID);
				end
			end
		end
	end

	return currencies, items;
end

-------------------------------------------------------------------------------------------------------------------------
local currencyFrame = CreateFrame("Frame", nil, MerchantFrame);
currencyFrame.entries = {};	--	index : entry

local entryPool = {};
local ENTRY_GAP = 5;
local NUM_ROW = 2;

function currencyFrame:init()
	self:UnregisterAllEvents();
	self:SetPoint("BOTTOM", MerchantFrame ,"TOP");
	self:SetPoint("LEFT", _G["MerchantFramePortrait"], "RIGHT");
	self:SetPoint("RIGHT");
	self:SetHeight(40 + 2 * ENTRY_GAP);
	self:SetBackdrop(BACKDROP);
	self:SetBackdropColor(0, 0, 0, 1);

	self._Hide = self.Hide;
	self:SetScript("OnHide", function(self)
		self:Clear();
	end);
end

function currencyFrame:Clear()
	for _, entry in ipairs(self.entries) do
		entry.id = nil;
		entry.icon:SetDesaturated(false);
		entry:ClearAllPoints();
		entry:Hide();
		tinsert(entryPool, entry);
	end
	wipe(self.entries);
end

function currencyFrame:addEntrey(id, texture, link, isCurrency)
	local entry = tremove(entryPool);
	if not entry then
		entry = CreateFrame("Button", nil, self);
		entry:SetHeight(20);
		entry:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");
		entry.icon = entry:CreateTexture(nil, "ARTWORK");
		entry.icon:SetPoint("LEFT");
		entry.icon:SetWidth(18);
		entry.icon:SetHeight(18);
		entry.label = entry:CreateFontString(nil, "ARTWORK", "SystemFont_Outline_Small");
		entry.label:SetPoint("RIGHT");
		entry.label:SetJustifyH("RIGHT");
		entry:EnableMouse(true);
		entry:SetScript("OnClick", function()
			PlaySoundFile(SOUNDS[random(1, #SOUNDS)]);
		end);
		entry:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
			if self.tip then
				GameTooltip:SetHyperlink(self.tip);
				GameTooltip:Show();
			end
		end);
		entry:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
	end
	tinsert(self.entries, entry);

	entry.id = id;
	entry.icon:SetTexture(texture);
	entry.tip = link;
	entry.isCurrency = isCurrency;

	entry:Show();
end

function currencyFrame:buildCurrencyEntries(currencies, items)
	if not self:IsShown() then return; end
	self:Clear();
	for _, currencyID in ipairs(currencies) do
		currencyFrame:addEntrey(currencyID, select(3, GetCurrencyInfo(currencyID)), GetCurrencyLink(currencyID), true);
	end
	for _, itemID in ipairs(items) do
		local _, link, _, _, _, _, _, _, _, texture = GetItemInfo(itemID);
		currencyFrame:addEntrey(itemID, texture, link, false);
	end
end

function currencyFrame:layout(lv)
	if lv == 1 then
		local previous, colX = nil, 0;
		local index = 0;
		for _, entry in ipairs(self.entries) do
			entry:ClearAllPoints();
			if not previous then
				previous = entry;
				entry:SetWidth(entry.preferW);
				entry:SetPoint("TOPLEFT", ENTRY_GAP, -ENTRY_GAP);
			elseif index % NUM_ROW == 0 then
				colX = previous and (colX + previous:GetWidth() + 2 * ENTRY_GAP) or 0;
				previous = entry;
				entry:SetWidth(entry.preferW);
				entry:SetPoint("TOPLEFT", colX, -ENTRY_GAP);
			else
				local rowshift = (index % NUM_ROW) * -20;
				entry:SetPoint("TOPLEFT", previous, "TOPLEFT", 0, rowshift);
				entry:SetPoint("TOPRIGHT", previous, "TOPRIGHT", 0, rowshift);
				previous:SetWidth(math.max(entry.preferW, previous:GetWidth()));
			end
			index = index + 1;
			if (entry:GetRight() + ENTRY_GAP) > self:GetRight() then
				return true;
			end
		end
	elseif lv == 2 then
		local previous, colX = nil, 0;
		local lc, rc = 0, 0;
		for _, entry in ipairs(self.entries) do
			entry:ClearAllPoints();
			local count = tonumber(entry.label:GetText());
			if not count or count == 0 then
				entry.icon:SetDesaturated(true);
				entry.label:SetText();
				entry:SetWidth(20);
				entry:SetPoint("BOTTOMRIGHT" ,floor(rc / NUM_ROW) * -20 - ENTRY_GAP ,(rc % NUM_ROW) * 20 + ENTRY_GAP);
				rc = rc + 1;
			else
				if not previous then
					previous = entry;
					entry:SetWidth(entry.preferW);
					entry:SetPoint("TOPLEFT", ENTRY_GAP, -ENTRY_GAP);
				elseif lc % NUM_ROW == 0 then
					colX = colX + previous:GetWidth() + 2 * ENTRY_GAP;
					previous = entry;
					entry:SetWidth(entry.preferW);
					entry:SetPoint("TOPLEFT", colX, -ENTRY_GAP);
				else
					local rowshift = (lc % NUM_ROW) * -20;
					entry:SetPoint("TOPLEFT", previous, "TOPLEFT", 0, rowshift);
					entry:SetPoint("TOPRIGHT", previous, "TOPRIGHT", 0, rowshift);
					previous:SetWidth(math.max(entry.preferW, previous:GetWidth()));
				end
				lc = lc + 1;
			end
		end
		local totalW = previous and previous:GetWidth() or 0;
		totalW = totalW + colX + floor(rc / NUM_ROW) * 20 + 2 * ENTRY_GAP;
		if totalW > self:GetWidth() then
			self:SetPoint("RIGHT", totalW -self:GetWidth() -40, 0);
		end
	end
	return false;
end

function currencyFrame:update()
	if not self:IsShown() then return; end
	for _, entry in ipairs(self.entries) do
		if entry.isCurrency then
			entry.label:SetText(select(2, GetCurrencyInfo(entry.id)));
		else
			entry.label:SetText(GetItemCount(entry.id));
		end
		entry.preferW = ceil(entry.icon:GetWidth() + entry.label:GetStringWidth() + ENTRY_GAP);
	end
	local compactLevel = 1;
	while currencyFrame:layout(compactLevel) do
		compactLevel = compactLevel + 1;
	end
end

currencyFrame:init();
currencyFrame:Hide();

-------------------------------------------------------------------------------------------------------------------------
local addon = CreateFrame("Frame", nil, UIParent);
addon:UnregisterAllEvents();
addon:Hide();

function addon:PLAYER_LOGIN()
	self:UnregisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_CLOSED");

	hooksecurefunc("SetMerchantFilter", function(filterID)	--MERCHANT_FILTER_ITEM_UPDATE sucks..
		self:MERCHANT_UPDATE();
	end);
end

function addon:MERCHANT_SHOW()
	--self:RegisterEvent("MERCHANT_FILTER_ITEM_UPDATE");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
	self:RegisterEvent("MERCHANT_UPDATE");
	self:RegisterEvent("BAG_UPDATE");
	self:MERCHANT_UPDATE();
end

function addon:MERCHANT_CLOSED()
	--self:UnregisterEvent("MERCHANT_FILTER_ITEM_UPDATE");
	self:UnregisterEvent("CURRENCY_DISPLAY_UPDATE");
	self:UnregisterEvent("MERCHANT_UPDATE");
	self:UnregisterEvent("BAG_UPDATE");
	currencyFrame:Hide();
end

function addon:MERCHANT_UPDATE()
	if not _G["MerchantFrame"]:IsShown() then return; end

	local currencies, items = _scanMerchantCurrency();
	if #currencies == 0 and #items == 0 then
		currencyFrame:Hide();
	else
		currencyFrame:Show();
		currencyFrame:buildCurrencyEntries(currencies, items);
		currencyFrame:update();
	end
end

function addon:MERCHANT_FILTER_ITEM_UPDATE()
	self:MERCHANT_UPDATE();
end

function addon:BAG_UPDATE()
	currencyFrame:update();
end

function addon:CURRENCY_DISPLAY_UPDATE()
	currencyFrame:update();
end

addon:SetScript("OnEvent", function(self, event, ...)
	local f = self[event];
	if f then
		f(self,...);
	else
		am(tostring(event));
	end
end);

addon:RegisterEvent("PLAYER_LOGIN");

-------------------------------------------------------------------------------------------------------------------------
